<!--- 
	csw-slider-menu.cfc 
	
	Built from CommonSpot Navigation Tree Element Data.
	
	Version: [2018-01-02 01:28 pm]
---> 

<cfcomponent displayname="csw-slider-menu" output="false" hint="Slider Menu Functions" extends="_cs_widgets._common.csw-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			Server.CommonSpot.udf.resources.loadResources("JQuery");
			Server.CommonSpot.udf.resources.loadResources("Bootstrap");
			Server.CommonSpot.udf.resources.loadResources("FontAwesome");
			Server.CommonSpot.udf.resources.loadResources("Pushy");
			Server.CommonSpot.udf.resources.loadResources("cswCommonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswButtonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswTextStyles");

			/* stylesheet for some basic styling of menu button and menu */
			Server.CommonSpot.udf.resources.loadResources("cswSliderMenuCSS");

			// If a custom stylesheets have been registered, go ahead and load them.
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCommonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCommonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswButtonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswButtonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTextStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTextStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswSliderMenuCSS_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswSliderMenuCSS_custom");
		</cfscript>
	</cffunction>
	
	
	
	<cffunction name="loadProperties" access="public" returntype="struct" output="Yes">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">
		
		<cfscript>
			var widgetMetadataFormName = "CSW Slider Menu";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();
			
			
			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;

			
			// 
			// Get Metadata (Core Slider Menu metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];
		
		
			/**************************************/
			/**** SLIDER CONFIG PROPERTIES TAB ****/
			/**************************************/
			
			// Slider Position
			props.sliderPosition = getStandardField( widgetData=widgetData, field='Slider_Position', defValue='pushy-left' );

			// Slider Width
			props.sliderWidth = getStandardField( widgetData=widgetData, field='Slider_Width', defValue='300' );

			/***********************************/
			/**** MENU STYLE PROPERTIES TAB ****/
			/***********************************/

			// Menu Styles
			props.menuStyles = getFieldSettings(widgetData=widgetData, field='menuStyles', defColor='cswBgColor-lightsteelblue', defBgColor='cswTcColor-white', defBorderColor='cswHoverColor-black', defStyle="dark");
			props.menuStyles.hoverColor = props.menuStyles.borderColor;

			// Enable Parent Links
			props.parentLinks = getStandardField( widgetData=widgetData, field='parentLinks', defValue='yes' );

			// Include Container
			props.includeContainer = getStandardField( widgetData=widgetData, field='Include_Container', defValue='no' );
			
			// Container Margin
			props.containerMargin = getStandardField( widgetData=widgetData, field='Container_Margin', defValue='no' );
			
			
			/************************************/
			/**** OPEN BUTTON PROPERTIES TAB ****/
			/************************************/

			// Open Button Display
			// props.openButtonDisplay = getStandardField( widgetData=widgetData, field='Open_Button_Display', defValue='show' );

			// Open Button Text
			props.buttonType = getStandardField( widgetData=widgetData, field='Button_Type', defValue='cswButton' );

			// Open Button Text
			props.openButtonText = getStandardField( widgetData=widgetData, field='Open_Button_Text', defValue='Menu' );

			// Open Link/Button Style 			
			props.button = getFieldSettings(widgetData=widgetData, field='Button', defShow='1', defAlign='cswTextAlign-inherit', defSize='cswSize-normal', defColor='white', defBgColor="black", defStyle="", defEffects="cswEffect-default", defPadding="");

			// Open Button Corners
			props.buttonCorners = getStandardField( widgetData=widgetData, field='Button_Corners', defValue='cswBtn-rounded' );

			// Link Style
			props.link = getFieldSettings(widgetData=widgetData, field='Link', defShow='0', defAlign='', defSize='', defColor="", defPadding="");

			// Link/Button Icon
			props.openButtonIcon = getStandardField( widgetData=widgetData, field='Open_Button_Icon', defValue='glyphicon-menu-hamburger' );

			
			/*************************************/
			/**** CLOSE BUTTON PROPERTIES TAB ****/
			/*************************************/

			// Close Button Display
			// props.closeButtonDisplay = getStandardField( widgetData=widgetData, field='Close_Button_Display', defValue='show' );

			// Close Link/Button Type
			props.closeButtonType = getStandardField( widgetData=widgetData, field='Close_Button_Type', defValue='cswButton' );

			// Open Button Text
			props.closeButtonText = getStandardField( widgetData=widgetData, field='Close_Button_Text', defValue='Close' );
			
			// Close Button Button Style
			props.closeButton = getFieldSettings(widgetData=widgetData, field='Close_Button', defShow='1', defAlign='cswTextAlign-inherit', defSize='cswSize-normal', defColor='white', defBgColor="black", defStyle="", defEffects="cswEffect-default", defPadding="");

			// Close Button Corners
			props.closeButtonCorners = getStandardField( widgetData=widgetData, field='Close_Button_Corners', defValue='cswBtn-rounded' );

			// Close Link Style
			props.closeLink = getFieldSettings(widgetData=widgetData, field='Close_Link', defShow='0', defAlign='', defSize='', defColor="", defPadding="");

			// Close Link/Button Icon
			props.closeButtonIcon = getStandardField( widgetData=widgetData, field='Close_Button_Icon', defValue='' );

			return props;
		</cfscript>
	</cffunction>
	
	
	
	<cffunction name="getAndRenderItems" access="public" output="Yes" returntype="void">
		<cfargument name="elementDataStruct" type="struct" required="Yes">
		<cfargument name="elementParamsStruct" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
	
		<cfscript>
			var elementData = arguments.elementDataStruct;
			var props = arguments.propsStruct;
			var data = StructNew();
			var totalItems = 0;

			var treeNodes = elementData.treeNodes;
			var eParams = arguments.elementParamsStruct;
			var nodeParams = StructNew();
			var styleAttr = '';
			var sliderMenuInLineCSS = '';
						
			nodeParams.curDepth = 0;

			// Check if the tree config is to surround the node description with a specific tag
			if( eParams.surroundDescriptionTag neq "" AND eParams.surroundDescriptionTag neq "none")
				nodeParams.DescriptionTag = '<#eParams.surroundDescriptionTag#>[]</#eParams.surroundDescriptionTag#>';
			else
				nodeParams.DescriptionTag = '[]';

			nodeParams.beginItem = '<li class="[]">';
			nodeParams.endItem = '</li>';
			nodeParams.beginList = '<ul class="[c1]">';
			nodeParams.endList = '</ul>';

			// Get Records from the Element as an Array of Structures
			if( StructKeyExists(elementData, 'treeNodes') AND arrayLen(elementData.treeNodes) )
				totalItems = arrayLen(elementData.treeNodes);
		</cfscript>
		
		<cfif totalItems>

			<cfscript>
				if( props.buttonType neq 'none')
				{
					props.linktext = props.openButtonText;
					props.linkURL = "##";
					props.button.fontClasses = "pushy-link";
					props.link.fontClasses = "pushy-link";
					props.linkAttrStruct = StructNew();
					props.buttonIcon = props.openButtonIcon;

					// Render Open Button
					// ---------------------
					RenderOpenButton(props);
				}
				
				// for slider menu and close button...
				if( len(props.menuStyles.bgColor) )
					styleAttr = 'background-color:#props.menuStyles.bgColor#;';

				if( len(props.menuStyles.color) )
					styleAttr = styleAttr & 'color:#props.menuStyles.color#;';
					
				if( len(styleAttr) )
					styleAttr = 'style="padding:5px;#styleAttr#"';	

				props.linktext = props.closeButtonText;
				props.linkURL = "##";
				props.closeButton.fontClasses = "pushy-link";
				props.closeLink.fontClasses = "pushy-link";
				props.linkAttrStruct = StructNew();
				props.buttonIcon = props.closeButtonIcon;				
			</cfscript>

			<cfoutput>
				<div id="cswSliderMenu#request.element.ID#" class="cswSliderMenu">
					<nav class="pushy #props.sliderPosition#" #styleAttr# data-useparentlinks="#props.parentLinks#">
						#RenderCloseButton(props)#
						#renderContainerAbove(props)# 
						#ProcessNavTreeArray(treeNodes, props, eparams, nodeParams)#
						#renderContainerBelow(props)#					
					</nav>
					<div class="site-overlay site-overlay-#props.menuStyles.style#"></div>
				</div>
			</cfoutput>

			<!--- // Build the Slider Menu Inline CSS for the menu width --->
			<cfsavecontent variable="sliderMenuInLineCSS">
				<cfoutput>
					<cfif props.sliderWidth NEQ 200>
						/* Update the slider menu width to config menu width value  */
						.cswSliderMenu .pushy {
							width: #props.sliderWidth#px;
							max-width: 90vw;
						}

						.cswSliderMenu .pushy-left {
							transform: translate3d(-#props.sliderWidth#px,0,0);
						}
						.pushy-open-left .cswSliderMenu .pushy-left {
							transform: none;
						}

						.cswSliderMenu .pushy-right {
							transform: translate3d(#props.sliderWidth#px, 0, 0);
						}
						.pushy-open-right .cswSliderMenu .pushy-right {
							transform: none;
						}
						.cswSliderMenu .cswTitle:hover {
							color: #props.menuStyles.hoverColor#;
						}
						.cswSliderMenu .pushy a {
							color: #props.menuStyles.color#;
						}
					</cfif>
				</cfoutput>
			</cfsavecontent>
			
			<cfscript>
				// Load the Slider Menu Inline CSS
				Server.CommonSpot.udf.resources.addHeaderCSS(sliderMenuInLineCSS, "TERTIARY");
			</cfscript>

		</cfif>
	
	</cffunction>
	
	

	<!--- // Build from CommonSpot Navigation Tree Element Data --->
	<cffunction name="ProcessNavTreeArray" access="private" output="Yes">
		<cfargument name="arr" type="array" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="eparams" type="struct" required="Yes">
		<cfargument name="nodeParams" type="struct" required="Yes">

			<cfscript>
				var i = 0;
				var props = arguments.propsStruct;
				var data = StructNew();
				var beginListTag = '';
				
				local.nodeParams = arguments.nodeParams;
				local.nodeParams.curDepth = local.nodeParams.curDepth + 1;
				
				beginListTag = ReplaceNoCase( local.nodeParams.beginList, ' class="[c1]"', '' );
				beginListTag = ReplaceNoCase( beginListTag, ' class=""', '' );

				if( local.nodeParams.curDepth eq 1 )
				{
					writeOutput('<div class="pushy-content"><ul>');
				}
				else
				{
					WriteOutput( beginListTag ); // <ul>
				}

				for(i=1; i lte ArrayLen(arguments.arr); i = i + 1)
				{
					ProcessNode(arguments.arr[i], props, arguments.eparams, local.nodeParams);
				}

				if( local.nodeParams.curDepth eq 1)
				{
					writeOutput('</ul></div>');
				}
				else
				{
					WriteOutput( arguments.nodeParams.endList ); // </ul>
				}

				local.nodeParams.curDepth = local.nodeParams.curDepth - 1;
			</cfscript>

	</cffunction>
	
	

	<!--- Process Node ---->
	<cffunction name="ProcessNode" access="private" output="Yes">
		<cfargument name="node" type="struct" required="Yes">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="eparams" type="struct" required="Yes">
		<cfargument name="nodeParams" type="struct" required="Yes">

		<cfscript>
			var props = arguments.propsStruct;
			var nodeClasses = arguments.node['ItemClass'];
			var curPageURL = Request.CGIVars.script_name;
			var curNodeURL = arguments.node.LinkTarget.ResolvedURL.ServerRelative;

			if( ArrayLen(arguments.node['ChildNodes']) gt 0 )
				nodeClasses = ListAppend( nodeClasses, "pushy-submenu", " " );
			else
				nodeClasses = ListAppend( nodeClasses, "pushy-link", " " );

			if( arguments.eParams.includeBranchLeafClasses eq 1 )
			{
				if( ArrayLen(arguments.node['ChildNodes']) gt 0 )
					nodeClasses = ListAppend( nodeClasses, "branch", " " );
				else
					nodeClasses = ListAppend( nodeClasses, "leaf", " " );
			}

			if( curNodeURL EQ curPageURL)
				nodeClasses = ListAppend(nodeClasses, "CS_CurrentPageNode", " ");

			RenderNode(arguments.node, nodeClasses, props, arguments.eparams, arguments.nodeParams);
		</cfscript>
	</cffunction>
	
	

	<!--- LI/DIV Item Node rendering --->
	<cffunction name="RenderNode" access="private">
		<cfargument name="node" required="Yes" type="struct">
		<cfargument name="nodeClasses" required="Yes" type="string">
		<cfargument name="propsStruct" type="struct" required="Yes">
		<cfargument name="eparams" type="struct" required="Yes">
		<cfargument name="nodeParams" type="struct" required="Yes">

		<cfscript>
			var props = arguments.propsStruct;
			var parentNodeLinkHTML = '';
			var parentNodelinkTitle = "Click to open/close sub-menu.";
			var beginItemTag = '';
			var img = '';
			// var subNodelinkTitle="Click to follow the link.";

			if( props.parentLinks eq 'yes' )
				parentNodelinkTitle = "Click to open/close sub-menu. Double click to follow the link.";

			beginItemTag = ReplaceNoCase(arguments.nodeParams.beginItem, ' class="[]"', "");

			if( arguments.nodeClasses neq "")
				beginItemTag = ReplaceNoCase(arguments.nodeParams.beginItem, "[]", arguments.nodeClasses);

			WriteOutput(beginItemTag);
		</cfscript>

		<!--- Image --->
		<cfif arguments.eParams.includeImage eq 1 AND arguments.node['ImageURL'] neq ''>
			<cfscript>
				img = '<img src="#arguments.node['ImageServerURL']#" height="#arguments.node['ImageHeight']#" width="#arguments.node['ImageWidth']#" alt="#arguments.node['Title']#" title="#arguments.node['Title']#" />';
			</cfscript>

			<cfmodule template="/commonspot/utilities/ct-render-text.cfm"
				classList="#arguments.node['ItemClass']#"
				useStyleSheets="#Request.Page.UseStyleSheets#"
				wantDIV="0"
				textBody='<img src="#arguments.node['ImageServerURL']#" height="#arguments.node['ImageHeight']#" width="#arguments.node['ImageWidth']#" alt="#arguments.node['Title']#" title="#arguments.node['Title']#" />'
				linkTarget="#arguments.node['LinkTarget']['ResolvedURL']['ServerRelative']#"
				onClick="#arguments.node['LinkTarget']['EventHandlers']['Click']#"
				mouseOver="#arguments.node['LinkTarget']['EventHandlers']['MouseOver']#">
		</cfif>

		<!--- Title --->
		<cfif arguments.eParams.includeTitle eq 1 AND arguments.node['Title'] neq ''> <!--- AND arguments.node['ParentNodeID'] neq 0--->
			<cfscript>
				if( arguments.eParams.surroundTitleTag neq "" AND arguments.eParams.surroundTitleTag neq "none")
					WriteOutput("<#arguments.eParams.surroundTitleTag#>");
			</cfscript>

			<cfif ArrayLen(arguments.node['ChildNodes']) gt 0 >
				<cfsavecontent variable="parentNodeLinkHTML">
					<cfmodule template="/commonspot/utilities/ct-render-text.cfm"
						classList="cswTitle haschildren"
						useStyleSheets="#Request.Page.UseStyleSheets#"
						wantDIV="0"
						textBody="#arguments.node['Title']#"
						linkTarget="#arguments.node['LinkTarget']['ResolvedURL']['ServerRelative']#"
						onClick="#arguments.node['LinkTarget']['EventHandlers']['Click']#"
						mouseOver="#arguments.node['LinkTarget']['EventHandlers']['MouseOver']#"
						readBubbleText="#parentNodelinkTitle#">
				</cfsavecontent>

				<cfscript>
					WriteOutput(parentNodeLinkHTML);
				</cfscript>

			<cfelse>
				<cfmodule template="/commonspot/utilities/ct-render-text.cfm"
					classList="cswTitle nochildren"
					useStyleSheets="#Request.Page.UseStyleSheets#"
					wantDIV="0"
					textBody="#arguments.node['Title']#"
					linkTarget="#arguments.node['LinkTarget']['ResolvedURL']['ServerRelative']#"
					onClick="#arguments.node['LinkTarget']['EventHandlers']['Click']#"
					mouseOver="#arguments.node['LinkTarget']['EventHandlers']['MouseOver']#"
					readBubbleText="">
			</cfif>

			<cfscript>
				if( arguments.eParams.surroundTitleTag neq "" AND arguments.eParams.surroundTitleTag neq "none")
					WriteOutput("</#arguments.eParams.surroundTitleTag#>");
			</cfscript>
		</cfif>

		<!--- Description --->
		<cfif arguments.eParams.includeDescription eq 1 AND arguments.node['Description'] neq ''>
			<cfoutput>#ReplaceNoCase( arguments.nodeParams.DescriptionTag, '[]', arguments.node['Description'] )#</cfoutput>
		</cfif>

		<!--- Render any children --->
		<cfscript>
			if( ArrayLen(arguments.node['ChildNodes']) gt 0 )
			{
				if( arguments.eParams.MaxDepth eq 0 OR arguments.nodeParams.curDepth lt arguments.eParams.maxDepth )
				{
					ProcessNavTreeArray(arguments.node['ChildNodes'], props, arguments.eparams, arguments.nodeParams);
				}
			}
		</cfscript>

		<cfoutput>#arguments.nodeParams.endItem#</cfoutput>
	</cffunction>


	
	<cffunction name="RenderOpenButton" access="private" output="true" returntype="void">
		<cfargument name="propsStruct" type="struct" required="true">

		<cfscript>
			var props = arguments.propsStruct;
			
			// Render Open Button
			// ----------------------					
			writeoutput('<div class="cswOpenBtn #props.button.align#">');

			if( FindNoCase('cswButton', props.buttonType) )
			{
				renderButtonCore(
					type = props.buttonType,
					linkText = props.openButtonText,
					linkURL = props.linkURL,
					buttonStruct = props.button,
					buttonCorners = props.buttonCorners,
					icon = props.openButtonIcon,
					linkAttrStruct = props.linkAttrStruct );
			}
			else if( FindNoCase('cswLink', props.buttonType) )
			{
				renderTextCore(
					class = 'cswLinkWrapper',
					tag = 'div',
					text = props.openButtonText,
					color = props.link.color,
					backgroundColor = props.link.bgColor,
					size = props.link.size,
					align = props.link.align,
					fontClasses = props.link.fontClasses,
					padding = props.link.padding,
					effect = props.link.effect,
					linkURL = props.linkURL,
					linkClass = props.buttonType,
					iconClass = props.openButtonIcon,
					style = props.link.style,
					linkAttrStruct = props.linkAttrStruct );
			}
			writeoutput('</div>');
		</cfscript>

	</cffunction>



	<cffunction name="RenderCloseButton" access="private" output="true" returntype="void">
		<cfargument name="propsStruct" type="struct" required="true">

		<cfscript>
			var props = arguments.propsStruct;
			
			// Render the close button
			// --------------------------
			if( props.closeButtonType neq 'none' ) 
			{
				if( FindNoCase('cswButton', props.closeButtonType) )
				{
					renderButtonCore(
						type = props.closeButtonType,
						linkText = props.closeButtonText,
						linkURL = props.linkURL,
						buttonStruct = props.closeButton,
						buttonCorners = props.closeButtonCorners,
						icon = props.buttonIcon,
						linkAttrStruct = props.linkAttrStruct);
				}
				else if( FindNoCase('cswLink', props.closeButtonType) )
				{
					renderTextCore(
						class = 'cswLinkWrapper',
						tag = 'div',
						text = props.closeButtonText,
						color = props.closeLink.color,
						backgroundColor = props.closeLink.bgColor,
						size = props.closeLink.size,
						align = props.closeLink.align,
						fontClasses = props.closeLink.fontClasses,
						padding = props.closeLink.padding,
						effect = props.closeLink.effect,
						linkURL = props.linkURL,
						linkClass = props.closeButtonType,
						iconClass = props.closeButtonIcon,
						style = props.closeLink.style,
						linkAttrStruct = props.linkAttrStruct);
				}
			}
		</cfscript>
		
	</cffunction>



	<cffunction name="renderContainerAbove" access="private" output="true" returntype="void">
		<cfargument name="propsStruct" type="struct" required="true">

		<cfscript>
			var props = arguments.propsStruct;
			var styleAttr = '';
			
			if( len(props.containerMargin) )
				styleAttr = 'style="margin:#props.containerMargin#;"';
		</cfscript>
		
		<cfif props.includeContainer eq 'above' OR props.includeContainer eq 'both'>
			<cfoutput><div class="cswContainerAbove" #StyleAttr#></cfoutput>
			
			<CFMODULE TEMPLATE="/commonspot/utilities/ct-render-named-element.cfm"
								elementtype="container"
								elementName="sliderMenuContainer_above_#request.element.id#">

			<cfoutput></div></cfoutput>
		</cfif>		
	
	</cffunction>


	
	<cffunction name="renderContainerBelow" access="private" output="true" returntype="void">
		<cfargument name="propsStruct" type="struct" required="true">

		<cfscript>
			var props = arguments.propsStruct;
			var styleAttr = '';
			
			if( len(props.containerMargin) )
				styleAttr = 'style="margin:#props.containerMargin#;"';
		</cfscript>
		
		<cfif props.includeContainer eq 'below' OR props.includeContainer eq 'both'>
			<cfoutput><div class="cswContainerBelow" #StyleAttr#></cfoutput>
			
			<CFMODULE TEMPLATE="/commonspot/utilities/ct-render-named-element.cfm"
								elementtype="container"
								elementName="sliderMenuContainer_below_#request.element.id#">

			<cfoutput></div></cfoutput>
		</cfif>		
	
	</cffunction>

</cfcomponent>